import { Annotation, StateEffect, StateField, } from "@codemirror/state";
import * as automerge from "@automerge/automerge";
export const effectType = StateEffect.define({});
export function updateHeads(newHeads) {
    return effectType.of({ newHeads });
}
export function getLastHeads(state, field) {
    return state.field(field).lastHeads;
}
export function getPath(state, field) {
    return state.field(field).path;
}
export function plugin(doc, path) {
    return StateField.define({
        create() {
            return {
                lastHeads: automerge.getHeads(doc),
                unreconciledTransactions: [],
                path: path.slice(),
            };
        },
        update(value, tr) {
            const result = {
                lastHeads: value.lastHeads,
                unreconciledTransactions: value.unreconciledTransactions.slice(),
                path: path.slice(),
            };
            let clearUnreconciled = false;
            for (const effect of tr.effects) {
                if (effect.is(effectType)) {
                    result.lastHeads = effect.value.newHeads;
                    clearUnreconciled = true;
                }
            }
            if (clearUnreconciled) {
                result.unreconciledTransactions = [];
            }
            else {
                if (!isReconcileTx(tr)) {
                    result.unreconciledTransactions.push(tr);
                }
            }
            return result;
        },
    });
}
export const reconcileAnnotationType = Annotation.define();
export function isReconcileTx(tr) {
    return !!tr.annotation(reconcileAnnotationType);
}
export function makeReconcile(tr) {
    if (tr.annotations != null) {
        if (tr.annotations instanceof Array) {
            tr.annotations = [...tr.annotations, reconcileAnnotationType.of({})];
        }
        else {
            tr.annotations = [tr.annotations, reconcileAnnotationType.of({})];
        }
    }
    else {
        tr.annotations = [reconcileAnnotationType.of({})];
    }
    //return {
    //...tr,
    //annotations: reconcileAnnotationType.of({})
    //}
}
//# sourceMappingURL=plugin.js.map